@@ -29,34 +29,46 @@ module TkTextTagConfig
   end
   private :__item_pathname
 
+  def tag_cget_tkstring(tagOrId, option)
+    itemcget_tkstring(['tag', tagOrId], option)
+  end
   def tag_cget(tagOrId, option)
     itemcget(['tag', tagOrId], option)
   end
+  def tag_cget_strict(tagOrId, option)
+    itemcget_strict(['tag', tagOrId], option)
+  end
   def tag_configure(tagOrId, slot, value=None)
     itemconfigure(['tag', tagOrId], slot, value)
   end
   def tag_configinfo(tagOrId, slot=nil)
-    itemconfigure(['tag', tagOrId], slot)
+    itemconfiginfo(['tag', tagOrId], slot)
   end
   def current_tag_configinfo(tagOrId, slot=nil)
-    itemconfigure(['tag', tagOrId], slot)
+    current_itemconfiginfo(['tag', tagOrId], slot)
   end
 
+  def window_cget_tkstring(tagOrId, option)
+    itemcget_tkstring(['window', tagOrId], option)
+  end
   def window_cget(tagOrId, option)
     itemcget(['window', tagOrId], option)
   end
+  def window_cget_strict(tagOrId, option)
+    itemcget_strict(['window', tagOrId], option)
+  end
   def window_configure(tagOrId, slot, value=None)
     itemconfigure(['window', tagOrId], slot, value)
   end
   def window_configinfo(tagOrId, slot=nil)
-    itemconfigure(['window', tagOrId], slot)
+    itemconfiginfo(['window', tagOrId], slot)
   end
   def current_window_configinfo(tagOrId, slot=nil)
-    itemconfigure(['window', tagOrId], slot)
+    current_itemconfiginfo(['window', tagOrId], slot)
   end
 
-  private :itemcget, :itemconfigure
-  private :itemconfiginfo, :current_itemconfiginfo
+  private :itemcget_tkstring, :itemcget, :itemcget_strict
+  private :itemconfigure, :itemconfiginfo, :current_itemconfiginfo
 end
 
 class Tk::Text<TkTextWin
@@ -245,12 +257,16 @@ class Tk::Text<TkTextWin
 
   TkCommandNames = ['text'.freeze].freeze
   WidgetClassName = 'Text'.freeze
-  WidgetClassNames[WidgetClassName] = self
+  WidgetClassNames[WidgetClassName] ||= self
 
   def self.new(*args, &block)
     obj = super(*args){}
     obj.init_instance_variable
-    obj.instance_eval(&block) if defined? yield
+    if TkCore::WITH_RUBY_VM  ### Ruby 1.9 !!!!
+      obj.instance_exec(obj, &block) if defined? yield
+    else
+      obj.instance_eval(&block) if defined? yield
+    end
     obj
   end
 
@@ -260,14 +276,18 @@ class Tk::Text<TkTextWin
   end
 
   def __destroy_hook__
-    TkTextTag::TTagID_TBL.delete(@path)
-    TkTextMark::TMarkID_TBL.delete(@path)
+    TkTextTag::TTagID_TBL.mutex.synchronize{
+      TkTextTag::TTagID_TBL.delete(@path)
+    }
+    TkTextTag::TMarkID_TBL.mutex.synchronize{
+      TkTextMark::TMarkID_TBL.delete(@path)
+    }
   end
 
   def create_self(keys)
     #if keys and keys != None
     #  #tk_call_without_enc('text', @path, *hash_kv(keys, true))
-    #  tk_call_without_enc(self.class::TkCommandNames[0], @path, 
+    #  tk_call_without_enc(self.class::TkCommandNames[0], @path,
     #                     *hash_kv(keys, true))
     #else
     #  #tk_call_without_enc('text', @path)
@@ -292,7 +312,7 @@ class Tk::Text<TkTextWin
   end
 
   def index(idx)
-    Tk::Text::IndexString.new(tk_send_without_enc('index', 
+    Tk::Text::IndexString.new(tk_send_without_enc('index',
                                                   _get_eval_enc_str(idx)))
   end
 
@@ -332,7 +352,7 @@ class Tk::Text<TkTextWin
       || tag.kind_of?(TkTextWindow)
       tag.id
     else
-      tag  # maybe an Array of configure paramters
+      tag  # maybe an Array of configure parameters
     end
   end
   private :tagid
@@ -361,7 +381,7 @@ class Tk::Text<TkTextWin
 
   def mark_gravity(mark, direction=nil)
     if direction
-      tk_send_without_enc('mark', 'gravity', 
+      tk_send_without_enc('mark', 'gravity',
                           _get_eval_enc_str(mark), direction)
       self
     else
@@ -370,52 +390,74 @@ class Tk::Text<TkTextWin
   end
 
   def mark_set(mark, index)
-    tk_send_without_enc('mark', 'set', _get_eval_enc_str(mark), 
+    tk_send_without_enc('mark', 'set', _get_eval_enc_str(mark),
                         _get_eval_enc_str(index))
     self
   end
   alias set_mark mark_set
 
   def mark_unset(*marks)
-    tk_send_without_enc('mark', 'unset', 
+    tk_send_without_enc('mark', 'unset',
                         *(marks.collect{|mark| _get_eval_enc_str(mark)}))
     self
   end
   alias unset_mark mark_unset
 
   def mark_next(index)
-    tagid2obj(_fromUTF8(tk_send_without_enc('mark', 'next', 
+    tagid2obj(_fromUTF8(tk_send_without_enc('mark', 'next',
                                             _get_eval_enc_str(index))))
   end
   alias next_mark mark_next
 
   def mark_previous(index)
-    tagid2obj(_fromUTF8(tk_send_without_enc('mark', 'previous', 
+    tagid2obj(_fromUTF8(tk_send_without_enc('mark', 'previous',
                                             _get_eval_enc_str(index))))
   end
   alias previous_mark mark_previous
 
-  def image_cget(index, slot)
+  def image_cget_strict(index, slot)
     case slot.to_s
     when 'text', 'label', 'show', 'data', 'file'
-      _fromUTF8(tk_send_without_enc('image', 'cget', 
+      _fromUTF8(tk_send_without_enc('image', 'cget',
                                     _get_eval_enc_str(index), "-#{slot}"))
     else
-      tk_tcl2ruby(_fromUTF8(tk_send_without_enc('image', 'cget', 
-                                                _get_eval_enc_str(index), 
+      tk_tcl2ruby(_fromUTF8(tk_send_without_enc('image', 'cget',
+                                                _get_eval_enc_str(index),
                                                 "-#{slot}")))
     end
   end
 
+  def image_cget(index, slot)
+    unless TkItemConfigMethod.__IGNORE_UNKNOWN_CONFIGURE_OPTION__
+      image_cget_strict(index, slot)
+    else
+      begin
+        image_cget_strict(index, slot)
+      rescue => e
+        begin
+          if current_image_configinfo(index).has_key?(slot.to_s)
+            # not tag error & option is known -> error on known option
+            fail e
+          else
+            # not tag error & option is unknown
+            nil
+          end
+        rescue
+          fail e  # tag error
+        end
+      end
+    end
+  end
+
   def image_configure(index, slot, value=None)
     if slot.kind_of?(Hash)
-      _fromUTF8(tk_send_without_enc('image', 'configure', 
-                                    _get_eval_enc_str(index), 
+      _fromUTF8(tk_send_without_enc('image', 'configure',
+                                    _get_eval_enc_str(index),
                                     *hash_kv(slot, true)))
     else
-      _fromUTF8(tk_send_without_enc('image', 'configure', 
-                                    _get_eval_enc_str(index), 
-                                    "-#{slot}", 
+      _fromUTF8(tk_send_without_enc('image', 'configure',
+                                    _get_eval_enc_str(index),
+                                    "-#{slot}",
                                     _get_eval_enc_str(value)))
     end
     self
@@ -445,16 +487,16 @@ class Tk::Text<TkTextWin
           else
             if conf[3]
               if conf[3].index('{')
-                conf[3] = tk_split_list(conf[3]) 
+                conf[3] = tk_split_list(conf[3])
               else
-                conf[3] = tk_tcl2ruby(conf[3]) 
+                conf[3] = tk_tcl2ruby(conf[3])
               end
             end
             if conf[4]
               if conf[4].index('{')
-                conf[4] = tk_split_list(conf[4]) 
+                conf[4] = tk_split_list(conf[4])
               else
-                conf[4] = tk_tcl2ruby(conf[4]) 
+                conf[4] = tk_tcl2ruby(conf[4])
               end
             end
           end
@@ -486,16 +528,16 @@ class Tk::Text<TkTextWin
           else
             if conf[2]
               if conf[2].index('{')
-                conf[2] = tk_split_list(conf[2]) 
+                conf[2] = tk_split_list(conf[2])
               else
-                conf[2] = tk_tcl2ruby(conf[2]) 
+                conf[2] = tk_tcl2ruby(conf[2])
               end
             end
             if conf[3]
               if conf[3].index('{')
-                conf[3] = tk_split_list(conf[3]) 
+                conf[3] = tk_split_list(conf[3])
               else
-                conf[3] = tk_tcl2ruby(conf[3]) 
+                conf[3] = tk_tcl2ruby(conf[3])
               end
             end
           end
@@ -584,7 +626,7 @@ class Tk::Text<TkTextWin
   end
 
   def compare(idx1, op, idx2)
-    bool(tk_send_without_enc('compare', _get_eval_enc_str(idx1), 
+    bool(tk_send_without_enc('compare', _get_eval_enc_str(idx1),
                              op, _get_eval_enc_str(idx2)))
   end
 
@@ -607,7 +649,7 @@ class Tk::Text<TkTextWin
   def count_info(idx1, idx2, update=true)
     # Tk8.5 feature
     opts = [
-      :chars, :displaychars, :displayindices, :displaylines, 
+      :chars, :displaychars, :displayindices, :displaylines,
       :indices, :lines, :xpixels, :ypixels
     ]
     if update
@@ -701,8 +743,8 @@ class Tk::Text<TkTextWin
   end
 
   def tag_add(tag, index1, index2=None)
-    tk_send_without_enc('tag', 'add', _get_eval_enc_str(tag), 
-                        _get_eval_enc_str(index1), 
+    tk_send_without_enc('tag', 'add', _get_eval_enc_str(tag),
+                        _get_eval_enc_str(index1),
                         _get_eval_enc_str(index2))
     self
   end
@@ -710,17 +752,19 @@ class Tk::Text<TkTextWin
   alias add_tag tag_add
 
   def tag_delete(*tags)
-    tk_send_without_enc('tag', 'delete', 
+    tk_send_without_enc('tag', 'delete',
                         *(tags.collect{|tag| _get_eval_enc_str(tag)}))
-    if TkTextTag::TTagID_TBL[@path]
-      tags.each{|tag|
-        if tag.kind_of?(TkTextTag)
-          TkTextTag::TTagID_TBL[@path].delete(tag.id) 
-        else
-          TkTextTag::TTagID_TBL[@path].delete(tag) 
-        end
-      }
-    end
+    TkTextTag::TTagID_TBL.mutex.synchronize{
+      if TkTextTag::TTagID_TBL[@path]
+        tags.each{|tag|
+          if tag.kind_of?(TkTextTag)
+            TkTextTag::TTagID_TBL[@path].delete(tag.id)
+          else
+            TkTextTag::TTagID_TBL[@path].delete(tag)
+          end
+        }
+      end
+    }
     self
   end
   alias deltag tag_delete
@@ -769,7 +813,7 @@ class Tk::Text<TkTextWin
   def tag_cget(tag, key)
     case key.to_s
     when 'text', 'label', 'show', 'data', 'file'
-      tk_call_without_enc(@path, 'tag', 'cget', 
+      tk_call_without_enc(@path, 'tag', 'cget',
                           _get_eval_enc_str(tag), "-#{key}")
     when 'font', 'kanjifont'
       #fnt = tk_tcl2ruby(tk_send('tag', 'cget', tag, "-#{key}"))
@@ -795,14 +839,14 @@ class Tk::Text<TkTextWin
           || key['latinfont'] || key['asciifont'] )
         tagfont_configure(tag, key)
       else
-        tk_send_without_enc('tag', 'configure', _get_eval_enc_str(tag), 
+        tk_send_without_enc('tag', 'configure', _get_eval_enc_str(tag),
                             *hash_kv(key, true))
       end
 
     else
-      if  key == 'font' || key == :font || 
+      if  key == 'font' || key == :font ||
           key == 'kanjifont' || key == :kanjifont ||
-          key == 'latinfont' || key == :latinfont || 
+          key == 'latinfont' || key == :latinfont ||
           key == 'asciifont' || key == :asciifont
         if val == None
           tagfontobj(tag)
@@ -810,7 +854,7 @@ class Tk::Text<TkTextWin
           tagfont_configure(tag, {key=>val})
         end
       else
-        tk_send_without_enc('tag', 'configure', _get_eval_enc_str(tag), 
+        tk_send_without_enc('tag', 'configure', _get_eval_enc_str(tag),
                             "-#{key}", _get_eval_enc_str(val))
       end
     end
@@ -840,16 +884,16 @@ class Tk::Text<TkTextWin
           else
             if conf[3]
               if conf[3].index('{')
-                conf[3] = tk_split_list(conf[3]) 
+                conf[3] = tk_split_list(conf[3])
               else
-                conf[3] = tk_tcl2ruby(conf[3]) 
+                conf[3] = tk_tcl2ruby(conf[3])
               end
             end
             if conf[4]
               if conf[4].index('{')
-                conf[4] = tk_split_list(conf[4]) 
+                conf[4] = tk_split_list(conf[4])
               else
-                conf[4] = tk_tcl2ruby(conf[4]) 
+                conf[4] = tk_tcl2ruby(conf[4])
               end
             end
           end
@@ -888,16 +932,16 @@ class Tk::Text<TkTextWin
           else
             if conf[2]
               if conf[2].index('{')
-                conf[2] = tk_split_list(conf[2]) 
+                conf[2] = tk_split_list(conf[2])
               else
-                conf[2] = tk_tcl2ruby(conf[2]) 
+                conf[2] = tk_tcl2ruby(conf[2])
               end
             end
             if conf[3]
               if conf[3].index('{')
-                conf[3] = tk_split_list(conf[3]) 
+                conf[3] = tk_split_list(conf[3])
               else
-                conf[3] = tk_tcl2ruby(conf[3]) 
+                conf[3] = tk_tcl2ruby(conf[3])
               end
             end
           end
@@ -942,19 +986,19 @@ class Tk::Text<TkTextWin
 =end
 
   def tag_raise(tag, above=None)
-    tk_send_without_enc('tag', 'raise', _get_eval_enc_str(tag), 
+    tk_send_without_enc('tag', 'raise', _get_eval_enc_str(tag),
                         _get_eval_enc_str(above))
     self
   end
 
   def tag_lower(tag, below=None)
-    tk_send_without_enc('tag', 'lower', _get_eval_enc_str(tag), 
+    tk_send_without_enc('tag', 'lower', _get_eval_enc_str(tag),
                         _get_eval_enc_str(below))
     self
   end
 
   def tag_remove(tag, *indices)
-    tk_send_without_enc('tag', 'remove', _get_eval_enc_str(tag), 
+    tk_send_without_enc('tag', 'remove', _get_eval_enc_str(tag),
                         *(indices.collect{|idx| _get_eval_enc_str(idx)}))
     self
   end
@@ -963,7 +1007,7 @@ class Tk::Text<TkTextWin
     #l = tk_split_simplelist(tk_send_without_enc('tag', 'ranges',
     #                                            _get_eval_enc_str(tag)))
     l = tk_split_simplelist(tk_send_without_enc('tag', 'ranges',
-                                                _get_eval_enc_str(tag)), 
+                                                _get_eval_enc_str(tag)),
                             false, true)
     r = []
     while key=l.shift
@@ -973,18 +1017,18 @@ class Tk::Text<TkTextWin
   end
 
   def tag_nextrange(tag, first, last=None)
-    simplelist(tk_send_without_enc('tag', 'nextrange', 
-                                   _get_eval_enc_str(tag), 
-                                   _get_eval_enc_str(first), 
+    simplelist(tk_send_without_enc('tag', 'nextrange',
+                                   _get_eval_enc_str(tag),
+                                   _get_eval_enc_str(first),
                                    _get_eval_enc_str(last))).collect{|idx|
       Tk::Text::IndexString.new(idx)
     }
   end
 
   def tag_prevrange(tag, first, last=None)
-    simplelist(tk_send_without_enc('tag', 'prevrange', 
-                                   _get_eval_enc_str(tag), 
-                                   _get_eval_enc_str(first), 
+    simplelist(tk_send_without_enc('tag', 'prevrange',
+                                   _get_eval_enc_str(tag),
+                                   _get_eval_enc_str(first),
                                    _get_eval_enc_str(last))).collect{|idx|
       Tk::Text::IndexString.new(idx)
     }
@@ -994,7 +1038,7 @@ class Tk::Text<TkTextWin
   def window_cget(index, slot)
     case slot.to_s
     when 'text', 'label', 'show', 'data', 'file'
-      _fromUTF8(tk_send_without_enc('window', 'cget', 
+      _fromUTF8(tk_send_without_enc('window', 'cget',
                                     _get_eval_enc_str(index), "-#{slot}"))
     when 'font', 'kanjifont'
       #fnt = tk_tcl2ruby(tk_send('window', 'cget', index, "-#{slot}"))
@@ -1038,12 +1082,12 @@ class Tk::Text<TkTextWin
             slot['create'] = install_cmd(proc{_epath(p_create.call)})
           end
         end
-        tk_send_without_enc('window', 'configure', 
-                            _get_eval_enc_str(index), 
+        tk_send_without_enc('window', 'configure',
+                            _get_eval_enc_str(index),
                             *hash_kv(slot, true))
       else
         if slot == 'window' || slot == :window
-          # id = value 
+          # id = value
           # value = id.epath if id.kind_of?(TkWindow)
           value = _epath(value)
         end
@@ -1063,8 +1107,8 @@ class Tk::Text<TkTextWin
             value = install_cmd(proc{_epath(p_create.call)})
           end
         end
-        tk_send_without_enc('window', 'configure', 
-                            _get_eval_enc_str(index), 
+        tk_send_without_enc('window', 'configure',
+                            _get_eval_enc_str(index),
                             "-#{slot}", _get_eval_enc_str(value))
       end
     end
@@ -1091,16 +1135,16 @@ class Tk::Text<TkTextWin
           else
             if conf[3]
               if conf[3].index('{')
-                conf[3] = tk_split_list(conf[3]) 
+                conf[3] = tk_split_list(conf[3])
               else
-                conf[3] = tk_tcl2ruby(conf[3]) 
+                conf[3] = tk_tcl2ruby(conf[3])
               end
             end
             if conf[4]
               if conf[4].index('{')
-                conf[4] = tk_split_list(conf[4]) 
+                conf[4] = tk_split_list(conf[4])
               else
-                conf[4] = tk_tcl2ruby(conf[4]) 
+                conf[4] = tk_tcl2ruby(conf[4])
               end
             end
           end
@@ -1128,16 +1172,16 @@ class Tk::Text<TkTextWin
           else
             if conf[2]
               if conf[2].index('{')
-                conf[2] = tk_split_list(conf[2]) 
+                conf[2] = tk_split_list(conf[2])
               else
-                conf[2] = tk_tcl2ruby(conf[2]) 
+                conf[2] = tk_tcl2ruby(conf[2])
               end
             end
             if conf[3]
               if conf[3].index('{')
-                conf[3] = tk_split_list(conf[3]) 
+                conf[3] = tk_split_list(conf[3])
               else
-                conf[3] = tk_tcl2ruby(conf[3]) 
+                conf[3] = tk_tcl2ruby(conf[3])
               end
             end
           end
@@ -1193,11 +1237,11 @@ class Tk::Text<TkTextWin
 
     # $KCODE == 'NONE'
     if JAPANIZED_TK
-      tk_call_without_enc('kstring', 'length', 
+      tk_call_without_enc('kstring', 'length',
                           _get_eval_enc_str(txt)).to_i
     else
       begin
-        tk_call_without_enc('encoding', 'convertto', 'ascii', 
+        tk_call_without_enc('encoding', 'convertto', 'ascii',
                             _get_eval_enc_str(txt)).length
       rescue StandardError, NameError
         # sorry, I have no plan
@@ -1286,11 +1330,11 @@ class Tk::Text<TkTextWin
         pos = _ktext_length(txt[0..(pos-1)]) if pos > 0
         if pat.kind_of?(String)
           #return [index(start + " + #{pos} chars"), pat.split('').length]
-          return [index(start + " + #{pos} chars"), 
+          return [index(start + " + #{pos} chars"),
                   _ktext_length(pat), pat.dup]
         else
           #return [index(start + " + #{pos} chars"), $&.split('').length]
-          return [index(start + " + #{pos} chars"), 
+          return [index(start + " + #{pos} chars"),
                   _ktext_length(match), match]
         end
       else
@@ -1304,11 +1348,11 @@ class Tk::Text<TkTextWin
         pos = _ktext_length(txt[0..(pos-1)]) if pos > 0
         if pat.kind_of?(String)
           #return [index(start + " + #{pos} chars"), pat.split('').length]
-          return [index(start + " + #{pos} chars"), 
+          return [index(start + " + #{pos} chars"),
                   _ktext_length(pat), pat.dup]
         else
           #return [index(start + " + #{pos} chars"), $&.split('').length]
-          return [index(start + " + #{pos} chars"), 
+          return [index(start + " + #{pos} chars"),
                   _ktext_length(match), match]
         end
       else
@@ -1319,7 +1363,7 @@ class Tk::Text<TkTextWin
           pos = _ktext_length(txt[0..(pos-1)]) if pos > 0
           if pat.kind_of?(String)
             #return [index("1.0 + #{pos} chars"), pat.split('').length]
-            return [index("1.0 + #{pos} chars"), 
+            return [index("1.0 + #{pos} chars"),
                     _ktext_length(pat), pat.dup]
           else
             #return [index("1.0 + #{pos} chars"), $&.split('').length]
@@ -1409,7 +1453,7 @@ class Tk::Text<TkTextWin
       idx = str.index(/ /, i)
       result.push str[i..(idx-1)]
       i = idx + 1
-      
+
       # retrieve value
       case result[-1]
       when 'text'
@@ -1467,7 +1511,7 @@ class Tk::Text<TkTextWin
         break
       end
     end
-    
+
     kvis = []
     until result.empty?
       kvis.push [result.shift, result.shift, result.shift]
@@ -1532,7 +1576,8 @@ class Tk::Text<TkTextWin
 end
 
 #TkText = Tk::Text unless Object.const_defined? :TkText
-Tk.__set_toplevel_aliases__(:Tk, Tk::Text, :TkText)
+#Tk.__set_toplevel_aliases__(:Tk, Tk::Text, :TkText)
+Tk.__set_loaded_toplevel_aliases__('tk/text.rb', :Tk, Tk::Text, :TkText)
 
 
 #######################################
@@ -1549,7 +1594,8 @@ class Tk::Text::Peer < Tk::Text
 
   def create_self(keys)
     if keys and keys != None
-      tk_call_without_enc(@src_text.path, 'peer', 'create', @path)
+      tk_call_without_enc(@src_text.path, 'peer', 'create',
+                          @path, *hash_kv(keys, true))
     else
       tk_call_without_enc(@src_text.path, 'peer', 'create', @path)
     end
