@@ -403,7 +403,7 @@ module TestNetHTTP_version_1_1_methods
 
       conn = Net::HTTP.new('localhost', port)
       conn.read_timeout = 0.01
-      conn.open_timeout = 0.01
+      conn.open_timeout = 0.1
 
       th = Thread.new do
         assert_raise(Net::ReadTimeout) {
@@ -610,15 +610,15 @@ module TestNetHTTP_version_1_2_methods
 
   def test_set_form
     require 'tempfile'
-    file = Tempfile.new('ruby-test')
-    file << "\u{30c7}\u{30fc}\u{30bf}"
-    data = [
-      ['name', 'Gonbei Nanashi'],
-      ['name', "\u{540d}\u{7121}\u{3057}\u{306e}\u{6a29}\u{5175}\u{885b}"],
-      ['s"i\o', StringIO.new("\u{3042 3044 4e9c 925b}")],
-      ["file", file, filename: "ruby-test"]
-    ]
-    expected = <<"__EOM__".gsub(/\n/, "\r\n")
+    Tempfile.create('ruby-test') {|file|
+      file << "\u{30c7}\u{30fc}\u{30bf}"
+      data = [
+        ['name', 'Gonbei Nanashi'],
+        ['name', "\u{540d}\u{7121}\u{3057}\u{306e}\u{6a29}\u{5175}\u{885b}"],
+        ['s"i\o', StringIO.new("\u{3042 3044 4e9c 925b}")],
+        ["file", file, filename: "ruby-test"]
+      ]
+      expected = <<"__EOM__".gsub(/\n/, "\r\n")
 --<boundary>
 Content-Disposition: form-data; name="name"
 
@@ -638,13 +638,12 @@ Content-Type: application/octet-stream
 \xE3\x83\x87\xE3\x83\xBC\xE3\x82\xBF
 --<boundary>--
 __EOM__
-    start {|http|
-      _test_set_form_urlencoded(http, data.reject{|k,v|!v.is_a?(String)})
-      _test_set_form_multipart(http, false, data, expected)
-      _test_set_form_multipart(http, true, data, expected)
+      start {|http|
+        _test_set_form_urlencoded(http, data.reject{|k,v|!v.is_a?(String)})
+        _test_set_form_multipart(http, false, data, expected)
+        _test_set_form_multipart(http, true, data, expected)
+      }
     }
-  ensure
-    file.close! if file
   end
 
   def _test_set_form_urlencoded(http, data)
@@ -669,12 +668,12 @@ __EOM__
 
   def test_set_form_with_file
     require 'tempfile'
-    file = Tempfile.new('ruby-test')
-    file.binmode
-    file << $test_net_http_data
-    filename = File.basename(file.to_path)
-    data = [['file', file]]
-    expected = <<"__EOM__".gsub(/\n/, "\r\n")
+    Tempfile.create('ruby-test') {|file|
+      file.binmode
+      file << $test_net_http_data
+      filename = File.basename(file.to_path)
+      data = [['file', file]]
+      expected = <<"__EOM__".gsub(/\n/, "\r\n")
 --<boundary>
 Content-Disposition: form-data; name="file"; filename="<filename>"
 Content-Type: application/octet-stream
@@ -682,38 +681,36 @@ Content-Type: application/octet-stream
 <data>
 --<boundary>--
 __EOM__
-    expected.sub!(/<filename>/, filename)
-    expected.sub!(/<data>/, $test_net_http_data)
-    start {|http|
-      data.each{|k,v|v.rewind rescue nil}
-      req = Net::HTTP::Post.new('/')
-      req.set_form(data, 'multipart/form-data')
-      res = http.request req
-      body = res.body
-      header, _ = body.split(/\r\n\r\n/, 2)
-      assert_match(/\A--(?<boundary>\S+)/, body)
-      /\A--(?<boundary>\S+)/ =~ body
-      expected = expected.gsub(/<boundary>/, boundary)
-      assert_match(/^--(?<boundary>\S+)\r\n/, header)
-      assert_match(
-        /^Content-Disposition: form-data; name="file"; filename="#{filename}"\r\n/,
-        header)
-      assert_equal(expected, body)
+      expected.sub!(/<filename>/, filename)
+      expected.sub!(/<data>/, $test_net_http_data)
+      start {|http|
+        data.each{|k,v|v.rewind rescue nil}
+        req = Net::HTTP::Post.new('/')
+        req.set_form(data, 'multipart/form-data')
+        res = http.request req
+        body = res.body
+        header, _ = body.split(/\r\n\r\n/, 2)
+        assert_match(/\A--(?<boundary>\S+)/, body)
+        /\A--(?<boundary>\S+)/ =~ body
+        expected = expected.gsub(/<boundary>/, boundary)
+        assert_match(/^--(?<boundary>\S+)\r\n/, header)
+        assert_match(
+          /^Content-Disposition: form-data; name="file"; filename="#{filename}"\r\n/,
+          header)
+        assert_equal(expected, body)
 
-      data.each{|k,v|v.rewind rescue nil}
-      req['Transfer-Encoding'] = 'chunked'
-      res = http.request req
-      #assert_equal(expected, res.body)
+        data.each{|k,v|v.rewind rescue nil}
+        req['Transfer-Encoding'] = 'chunked'
+        res = http.request req
+        #assert_equal(expected, res.body)
+      }
     }
-  ensure
-    file.close! if file
   end
 end
 
 class TestNetHTTP_v1_2 < Test::Unit::TestCase
   CONFIG = {
     'host' => '127.0.0.1',
-    'port' => 0,
     'proxy_host' => nil,
     'proxy_port' => nil,
   }
@@ -731,7 +728,6 @@ end
 class TestNetHTTP_v1_2_chunked < Test::Unit::TestCase
   CONFIG = {
     'host' => '127.0.0.1',
-    'port' => 0,
     'proxy_host' => nil,
     'proxy_port' => nil,
     'chunked' => true,
@@ -762,7 +758,6 @@ end
 class TestNetHTTPContinue < Test::Unit::TestCase
   CONFIG = {
     'host' => '127.0.0.1',
-    'port' => 0,
     'proxy_host' => nil,
     'proxy_port' => nil,
     'chunked' => true,
@@ -847,7 +842,6 @@ end
 class TestNetHTTPKeepAlive < Test::Unit::TestCase
   CONFIG = {
     'host' => '127.0.0.1',
-    'port' => 0,
     'proxy_host' => nil,
     'proxy_port' => nil,
     'RequestTimeout' => 1,
@@ -899,7 +893,6 @@ end
 class TestNetHTTPLocalBind < Test::Unit::TestCase
   CONFIG = {
     'host' => 'localhost',
-    'port' => 0,
     'proxy_host' => nil,
     'proxy_port' => nil,
   }
@@ -923,7 +916,9 @@ class TestNetHTTPLocalBind < Test::Unit::TestCase
 
     http = Net::HTTP.new(config('host'), config('port'))
     http.local_host = Addrinfo.tcp(config('host'), config('port')).ip_address
-    http.local_port = [*10000..20000].shuffle.first.to_s
+    http.local_port = Addrinfo.tcp(config('host'), 0).bind {|s|
+      s.local_address.ip_port.to_s
+    }
     assert_not_nil(http.local_host)
     assert_not_nil(http.local_port)
 
