@@ -1979,6 +1979,15 @@ End
     }
   end
 
+  def test_reopen_stdio
+    mkcdtmpdir {
+      fname = 'bug11319'
+      File.write(fname, 'hello')
+      system(EnvUtil.rubybin, '-e', "STDOUT.reopen('#{fname}', 'w+')")
+      assert_equal('', File.read(fname))
+    }
+  end
+
   def test_reopen_mode
     feature7067 = '[ruby-core:47694]'
     make_tempfile {|t|
@@ -2896,7 +2905,7 @@ End
       }
 
       IO.select(tempfiles)
-  }, bug8080
+    }, bug8080, timeout: 30
   end
 
   def test_read_32bit_boundary
