@@ -4,6 +4,7 @@ rescue LoadError
 end
 
 require "test/unit"
+require_relative "../ruby/envutil"
 
 class TestSocketAddrinfo < Test::Unit::TestCase
   HAS_UNIXSOCKET = defined?(UNIXSocket) && /cygwin/ !~ RUBY_PLATFORM
@@ -140,7 +141,7 @@ class TestSocketAddrinfo < Test::Unit::TestCase
     ai = Addrinfo.new(s1.getsockname)
     s2 = Socket.new(:INET, :STREAM, 0)
     s2.connect(ai)
-    s3, sender_addr = s1.accept
+    s3, _ = s1.accept
     s2.send("test-socket-connect", 0)
     assert_equal("test-socket-connect", s3.recv(100))
   ensure
@@ -166,7 +167,7 @@ class TestSocketAddrinfo < Test::Unit::TestCase
       rescue Errno::EISCONN
       end
     end
-    s3, sender_addr = s1.accept
+    s3, _ = s1.accept
     s2.send("test-socket-connect-nonblock", 0)
     assert_equal("test-socket-connect-nonblock", s3.recv(100))
   ensure
@@ -468,6 +469,17 @@ class TestSocketAddrinfo < Test::Unit::TestCase
     assert_equal(ai1.canonname, ai2.canonname)
   end
 
+  def test_marshal_memory_leak
+    bug11051 = '[ruby-dev:48923] [Bug #11051]'
+    assert_no_memory_leak(%w[-rsocket], <<-PREP, <<-CODE, bug11051, rss: true)
+    d = Marshal.dump(Addrinfo.tcp("127.0.0.1", 80))
+    1000.times {Marshal.load(d)}
+    PREP
+    GC.start
+    20_000.times {Marshal.load(d)}
+    CODE
+  end
+
   if Socket.const_defined?("AF_INET6") && Socket::AF_INET6.is_a?(Integer)
 
     def test_addrinfo_new_inet6
